/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MatrixFunctions.h,v 1.1 2003/03/24 20:15:19 opetzold Exp $
 */

#ifndef TVMET_XPR_MATRIX_FUNCTIONS_H
#define TVMET_XPR_MATRIX_FUNCTIONS_H

namespace tvmet {

/* forwards */
template<class T, std::size_t Rows, std::size_t Cols> class Matrix;
template<class T, std::size_t Sz> class Vector;


/**
 * \fn product(XprMatrix<E1, Rows1, Cols1> lhs, XprMatrix<E2, Cols1, Cols2> rhs)
 * \brief Evaluate the product of two XprMatrix.
 * \ingroup _binary_function
 * This function is using temporaries. Expanding expressions with expressions
 * does increase the compile time dramaticly and the produced code slows
 * down for most cases.
 */
template<class E1, std::size_t Rows1, std::size_t Cols1,
	 class E2, std::size_t Cols2>
inline
XprMatrix<
  XprMMProduct<
    typename E1::value_type, Rows1, Cols1,
    typename E2::value_type, Cols2, Cols1,
    1, Cols2, 1
  >,
  Rows1, Cols2
>
product(XprMatrix<E1, Rows1, Cols1> lhs, XprMatrix<E2, Cols1, Cols2> rhs) {
  // precalculate the expressions to speed up at compile time and runtime!
  Matrix<typename E1::value_type, Rows1, Cols1>		tmp1( lhs );
  Matrix<typename E2::value_type, Cols2, Cols1>		tmp2( rhs );

  typedef XprMMProduct<
    typename E1::value_type, Rows1, Cols1,
    typename E2::value_type, Cols2, Cols1,
    1, Cols2, 1
  >							expr_type;
  return XprMatrix<expr_type, Rows1, Cols2>(expr_type(tmp1.data(), tmp2.data()));
}


/**
 * \fn product(XprMatrix<E1, Rows, Cols> lhs, XprVector<E2, Cols> rhs)
 * \brief Evaluate the product of XprMatrix and XprVector.
 * \ingroup _binary_function
 * This function is using temporaries. Expanding expressions with expressions
 * does increase the compile time dramaticly and the produced code slows
 * down for most cases.
 */
template<class E1, std::size_t Rows, std::size_t Cols,
	 class E2>
inline
XprVector<
  XprMVProduct<
    typename E1::value_type, Rows, Cols,
    Cols, 1, typename E2::value_type, 1
  >,
  Rows
>
product(XprMatrix<E1, Rows, Cols> lhs, XprVector<E2, Cols> rhs) {
  // precalculate the expressions to speed up at compile time and runtime!
  Matrix<typename E1::value_type, Rows, Cols>		tmp1( lhs );
  Vector<typename E2::value_type, Cols>			tmp2( rhs );

  typedef XprMVProduct<
    typename E1::value_type, Rows, Cols,
    Cols, 1, typename E2::value_type, 1
  >							expr_type;
  return XprVector<expr_type, Rows>(expr_type(tmp1.data(), tmp2.data()));
}

} // namespace tvmet

#endif // TVMET_XPR_MATRIX_FUNCTIONS_H

// Local Variables:
// mode:C++
// End:
